@@ -4,7 +4,7 @@
 Test cases related to XSLT processing
 """
 
-import unittest
+import unittest, copy
 
 from common_imports import etree, StringIO, HelperTestCase, fileInTestDir
 from common_imports import doctest
@@ -37,6 +37,54 @@ class ETreeXSLTTestCase(HelperTestCase):
     def test_xslt_input_none(self):
         self.assertRaises(TypeError, etree.XSLT, None)
 
+    if False and etree.LIBXSLT_VERSION >= (1,1,15):
+        # earlier versions generate no error
+        if etree.LIBXSLT_VERSION > (1,1,17):
+            def test_xslt_invalid_stylesheet(self):
+                style = self.parse('''\
+<xsl:stylesheet version="1.0"
+    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+  <xsl:stylesheet />
+</xsl:stylesheet>''')
+
+                self.assertRaises(
+                    etree.XSLTParseError, etree.XSLT, style)
+        
+    def test_xslt_copy(self):
+        tree = self.parse('<a><b>B</b><c>C</c></a>')
+        style = self.parse('''\
+<xsl:stylesheet version="1.0"
+    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+  <xsl:template match="*" />
+  <xsl:template match="/">
+    <foo><xsl:value-of select="/a/b/text()" /></foo>
+  </xsl:template>
+</xsl:stylesheet>''')
+
+        transform = etree.XSLT(style)
+        res = transform(tree)
+        self.assertEquals('''\
+<?xml version="1.0"?>
+<foo>B</foo>
+''',
+                          str(res))
+
+        transform_copy = copy.deepcopy(transform)
+        res = transform_copy(tree)
+        self.assertEquals('''\
+<?xml version="1.0"?>
+<foo>B</foo>
+''',
+                          str(res))
+
+        transform = etree.XSLT(style)
+        res = transform(tree)
+        self.assertEquals('''\
+<?xml version="1.0"?>
+<foo>B</foo>
+''',
+                          str(res))
+
     def test_xslt_utf8(self):
         tree = self.parse(u'<a><b>\uF8D2</b><c>\uF8D2</c></a>')
         style = self.parse('''\
@@ -91,12 +139,12 @@ class ETreeXSLTTestCase(HelperTestCase):
         st = etree.XSLT(style)
         res = st.apply(tree)
         expected = u"""\
-<?xml version='1.0' encoding='UTF-16'?>
+<?xml version='1.0' encoding='UTF-16'?>\
 <foo>\uF8D2</foo>"""
 
         f = StringIO()
-        res.write(f, 'UTF-16')
-        result = unicode(f.getvalue(), 'UTF-16')
+        res.write(f, encoding='UTF-16')
+        result = unicode(f.getvalue(), 'UTF-16').replace('\n', '')
         self.assertEquals(expected,
                           result)
 
@@ -120,17 +168,38 @@ class ETreeXSLTTestCase(HelperTestCase):
         self.assertEquals(expected,
                           unicode(res))
 
-    def test_exslt(self):
+    def test_exslt_str(self):
+        tree = self.parse('<a><b>B</b><c>C</c></a>')
+        style = self.parse('''\
+<xsl:stylesheet version="1.0"
+    xmlns:str="http://exslt.org/strings"
+    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
+    exclude-result-prefixes="str xsl">
+  <xsl:template match="text()">
+    <xsl:value-of select="str:align(string(.), '---', 'center')" />
+  </xsl:template>
+  <xsl:template match="*">
+    <xsl:copy>
+      <xsl:apply-templates/>
+    </xsl:copy>
+  </xsl:template>
+</xsl:stylesheet>''')
+
+        st = etree.XSLT(style)
+        res = st(tree)
+        self.assertEquals('''\
+<?xml version="1.0"?>
+<a><b>-B-</b><c>-C-</c></a>
+''',
+                          str(res))
+
+    def test_exslt_math(self):
         tree = self.parse('<a><b>B</b><c>C</c></a>')
         style = self.parse('''\
 <xsl:stylesheet version="1.0"
     xmlns:math="http://exslt.org/math"
-    xmlns:str="http://exslt.org/strings"
     xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
-    exclude-result-prefixes="math str xsl">
-  <xsl:template match="text()">
-    <xsl:value-of select="str:align(string(.), '---', 'center')" />
-  </xsl:template>
+    exclude-result-prefixes="math xsl">
   <xsl:template match="*">
     <xsl:copy>
       <xsl:attribute name="pi">
@@ -145,12 +214,11 @@ class ETreeXSLTTestCase(HelperTestCase):
         res = st(tree)
         self.assertEquals('''\
 <?xml version="1.0"?>
-<a pi="3.14"><b pi="3">-B-</b><c pi="3">-C-</c></a>
+<a pi="3.14"><b pi="3">B</b><c pi="3">C</c></a>
 ''',
-                          st.tostring(res))
+                          str(res))
 
     def test_xslt_input(self):
-        tree = self.parse('<a><b>B</b><c>C</c></a>')
         style = self.parse('''\
 <xsl:stylesheet version="1.0"
     xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
@@ -162,7 +230,6 @@ class ETreeXSLTTestCase(HelperTestCase):
 
         st = etree.XSLT(style)
         st = etree.XSLT(style.getroot())
-        self.assertRaises(TypeError, etree.XSLT, None)
 
     def test_xslt_input_partial_doc(self):
         style = self.parse('''\
@@ -209,11 +276,12 @@ class ETreeXSLTTestCase(HelperTestCase):
 ''',
                           st.tostring(res))
 
-    def _test_xslt_parameter_missing(self):
-        # DISABLED - NOT RELIABLE?
-        # apply() without needed parameter will lead to XSLTApplyError
-        tree = self.parse('<a><b>B</b><c>C</c></a>')
-        style = self.parse('''\
+    if etree.LIBXSLT_VERSION < (1,1,18):
+        # later versions produce no error
+        def test_xslt_parameter_missing(self):
+            # apply() without needed parameter will lead to XSLTApplyError
+            tree = self.parse('<a><b>B</b><c>C</c></a>')
+            style = self.parse('''\
 <xsl:stylesheet version="1.0"
     xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
   <xsl:template match="/">
@@ -221,9 +289,9 @@ class ETreeXSLTTestCase(HelperTestCase):
   </xsl:template>
 </xsl:stylesheet>''')
 
-        st = etree.XSLT(style)
-        self.assertRaises(etree.XSLTApplyError,
-                          st.apply, tree)
+            st = etree.XSLT(style)
+            self.assertRaises(etree.XSLTApplyError,
+                              st.apply, tree)
 
     def test_xslt_multiple_parameters(self):
         tree = self.parse('<a><b>B</b><c>C</c></a>')
@@ -554,7 +622,6 @@ class ETreeXSLTTestCase(HelperTestCase):
                           '{http://www.w3.org/1999/XSL/Transform}stylesheet')
 
     def test_xslt_document_error(self):
-        # make sure document('') works from parsed strings
         xslt = etree.XSLT(etree.XML("""\
 <xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
@@ -565,6 +632,68 @@ class ETreeXSLTTestCase(HelperTestCase):
 """))
         self.assertRaises(etree.XSLTApplyError, xslt, etree.XML('<a/>'))
 
+    def test_xslt_document_XML_resolver(self):
+        # make sure document('') works when custom resolvers are in use
+        assertEquals = self.assertEquals
+        called = {'count' : 0}
+        class TestResolver(etree.Resolver):
+            def resolve(self, url, id, context):
+                assertEquals(url, 'file://ANYTHING')
+                called['count'] += 1
+                return self.resolve_string('<CALLED/>', context)
+
+        parser = etree.XMLParser()
+        parser.resolvers.add(TestResolver())
+
+        xslt = etree.XSLT(etree.XML("""\
+<xsl:stylesheet version="1.0"
+   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
+   xmlns:l="local">
+  <xsl:template match="/">
+    <test>
+      <xsl:for-each select="document('')//l:data/l:entry">
+        <xsl:copy-of select="document('file://ANYTHING')"/>
+        <xsl:copy>
+          <xsl:attribute name="value">
+            <xsl:value-of select="."/>
+          </xsl:attribute>
+        </xsl:copy>
+      </xsl:for-each>
+    </test>
+  </xsl:template>
+  <l:data>
+    <l:entry>A</l:entry>
+    <l:entry>B</l:entry>
+  </l:data>
+</xsl:stylesheet>
+""", parser))
+
+        self.assertEquals(called['count'], 0)
+        result = xslt(etree.XML('<a/>'))
+        self.assertEquals(called['count'], 1)
+
+        root = result.getroot()
+        self.assertEquals(root.tag,
+                          'test')
+        self.assertEquals(len(root), 4)
+
+        self.assertEquals(root[0].tag,
+                          'CALLED')
+        self.assertEquals(root[1].tag,
+                          '{local}entry')
+        self.assertEquals(root[1].text,
+                          None)
+        self.assertEquals(root[1].get("value"),
+                          'A')
+        self.assertEquals(root[2].tag,
+                          'CALLED')
+        self.assertEquals(root[3].tag,
+                          '{local}entry')
+        self.assertEquals(root[3].text,
+                          None)
+        self.assertEquals(root[3].get("value"),
+                          'B')
+
     def test_xslt_move_result(self):
         root = etree.XML('''\
         <transform>
@@ -978,4 +1107,4 @@ def test_suite():
     return suite
 
 if __name__ == '__main__':
-    unittest.main()
+    print 'to test use test.py %s' % __file__
